package Tropical.Mathematics;

import java.util.ArrayList;

/**
 * 
 * @author alex
 * @author gooogleshandi@gmail.com
 */

public final class Evaluations {
	/**
	 * Class provides different evaluations between matrices, vectors and
	 * scalars.
	 */
	private int a;

	/**
	 * Function return a tropical trace of Matrix. Matrix can be only square
	 * (row count must be equal to column count).
	 * 
	 * @param mat
	 *            is the initial Matrix.
	 * @return returns a tropical trace of square Matrix.
	 */
	public static double getTrace(Matrix mat) {
		double tr = Double.NEGATIVE_INFINITY;
		for (int i = 0; i < mat.getColumnSize(); i++) {
			tr = Math.max(tr, mat.getValueAt(i, i));
		}
		return tr;
	}

	/**
	 * Function return a sum of tropical traces of Matrix A. Matrix can be only
	 * square (row count must be equal to column count).
	 * 
	 * @param mat
	 *            is the initial Matrix.
	 * @return returns a tropical trace of square Matrix.
	 */
	public static double getBigTrace(Matrix mat) {
		double result = Double.NEGATIVE_INFINITY;
		Matrix clone = Evaluations.matrixMultip(mat,
				Evaluations.getI(mat.getColumnSize()));
		for (int i = 0; i < mat.getColumnSize(); i++) {
			result = Math.max(result, getTrace(clone));
			clone = Evaluations.matrixMultip(clone, mat);
		}
		return result;
	}

	/**
	 * Returns a square matrix I of specific size n. Matrix I is a matrix with 0
	 * on it's diagonal and negative infinity elsewhere.
	 * 
	 * @param n
	 * @return matrix I.
	 */
	public static Matrix getI(int n) {
		Matrix i = new Matrix(n, n);
		for (int ij = 0; ij < n; ij++) {
			i.setValueAt(0, ij, ij);
		}
		return i;
	}

	public static Matrix getVectorI(int size) {
		Matrix i = new Matrix(1, size);
		for (int ij = 0; ij < size; ij++) {
			i.setValueAt(0, 0, ij);
		}
		return i;

	}

	/**
	 * Makes a tropical sum of several matrices.
	 * 
	 * @param array
	 *            array of matrices.
	 * @return returns a result as one matrix.
	 */
	public static Matrix arrayMatrixSum(ArrayList<Matrix> array) {
		Matrix result;
		result = matrixSum(array.get(0), array.get(1));
		for (int i = 2; i < array.size(); i++) {
			result = matrixSum(result, array.get(i));
		}
		return result;
	}

	/**
	 * Function resize the Matrix if it has last row or column fully equal to
	 * tropical zero.
	 * 
	 * @param first
	 *            initial Matrix, to be resized.
	 * @return return a Matrix with reduced row or column count. In limit
	 *         returns a one value(scalar).
	 */

	public static Matrix resizeMatrix(Matrix first) {
		boolean lastRowZero = false;
		boolean lastColumnZero = false;
		for (int i = 0; i < first.getColumnSize(); i++) {
			if (first.getValueAt(first.getRowSize() - 1, i) == Double.NEGATIVE_INFINITY) {
				lastRowZero = true;
			} else {
				lastRowZero = false;
				break;
			}
		}
		for (int i = 0; i < first.getRowSize(); i++) {
			if (first.getValueAt(i, first.getColumnSize() - 1) == Double.NEGATIVE_INFINITY) {
				lastColumnZero = true;
			} else {
				lastColumnZero = false;
				break;
			}
		}
		if (lastRowZero) {
			first.setRowSize(first.getRowSize() - 1, 0.0);
		}
		if (lastColumnZero) {
			first.setColumnSize(first.getColumnSize() - 1, 0.0);
		}
		if (lastColumnZero || lastRowZero) {
			first = resizeMatrix(first);
		}
		return first;

	}

	/**
	 * Makes a tropical multiplication of several matrices.
	 * 
	 * @param array
	 *            is an array of matrices.
	 * @return returns a result as one matrix.
	 */
	public static Matrix arrayMatrixMultip(ArrayList<Matrix> array) {
		Matrix result;
		result = matrixMultip(array.get(0), array.get(1));
		for (int i = 2; i < array.size(); i++) {
			result = matrixMultip(result, array.get(i));
		}
		return result;

	}

	/**
	 * Function provides a tropical multiplication of two matrices.
	 * 
	 * @param first
	 *            - left Matrix of operation
	 * @param second
	 *            - right Matrix of operation
	 * @return returns a result of tropical multiplication.
	 */
	public static Matrix matrixMultip(Matrix first, Matrix second) {
		if (second.getRowSize() == 1 && second.getColumnSize() == 1) {
			return matrixScalarMult(first, second.getValueAt(0, 0));
		}
		if (first.getRowSize() == 1 && first.getColumnSize() == 1) {
			return matrixScalarMult(second, first.getValueAt(0, 0));
		}

		if (second.getRowSize() != first.getColumnSize()) {
			System.out.println("Not valid matrices by row or column!");
			return null;
		}
		int cS = 0, rS = 0, kS = 0;
		cS = second.getColumnSize();
		rS = first.getRowSize();
		kS = first.getColumnSize();
		Matrix result = new Matrix(rS, cS);
		for (int i = 0; i < rS; i++) {
			for (int j = 0; j < cS; j++) {
				double sum = Double.NEGATIVE_INFINITY;
				for (int k = 0; k < kS; k++) {
					sum = Math.max(sum,
							first.getValueAt(i, k) + second.getValueAt(k, j));
				}
				result.setValueAt(sum, i, j);

			}

		}
		return result;
	}

	/**
	 * Makes a tropical sum of two matrices with double values.s
	 * 
	 * @param first
	 *            a first Matrix in operation.
	 * @param second
	 *            a second Matrix in operation.
	 * @return returns a result of tropical sum.
	 */
	public static Matrix matrixSum(Matrix first, Matrix second) {
		int rS = first.getRowSize() >= second.getRowSize() ? first.getRowSize()
				: second.getRowSize();
		int cS = first.getColumnSize() >= second.getColumnSize() ? first
				.getColumnSize() : second.getColumnSize();

		Matrix result = new Matrix(rS, cS);
		for (int i = 0; i < rS; i++) {
			for (int j = 0; j < cS; j++) {

				if (i >= first.getRowSize() && j < second.getColumnSize()) {
					result.setValueAt(second.getValueAt(i, j), i, j);
					continue;
				} else {
					if (i >= second.getRowSize() && j < first.getColumnSize()) {
						result.setValueAt(first.getValueAt(i, j), i, j);
						continue;
					}
				}
				if (j >= first.getColumnSize() && i < second.getRowSize()) {
					result.setValueAt(second.getValueAt(i, j), i, j);
					continue;
				} else {
					if (j >= second.getColumnSize() && i < first.getRowSize()) {
						result.setValueAt(first.getValueAt(i, j), i, j);
						continue;
					}
				}
				if (j < second.getColumnSize() && i < first.getRowSize()
						&& j < first.getColumnSize() && i < second.getRowSize()) {
					double maxi = Math.max(first.getValueAt(i, j),
							second.getValueAt(i, j));
					result.setValueAt(maxi, i, j);
				}
			}

		}
		return result;
	}

	/**
	 * Function makes a tropical sum with a Matrix.
	 * 
	 * @param first
	 *            a Matrix.
	 * @param scalar
	 *            a scalar.
	 * @return returns a Matrix after operation.
	 */
	public static Matrix matrixScalarSum(Matrix first, double scalar) {
		Matrix result = new Matrix(first.getRowSize(), first.getColumnSize());
		for (int i = 0; i < result.getRowSize(); i++) {
			for (int j = 0; j < result.getColumnSize(); j++) {
				result.setValueAt(Math.max(first.getValueAt(i, j), scalar), i,
						j);
			}
		}
		return result;
	}

	public static Matrix powerMatrix(Matrix m, int power) {
		if (power == 0)
			return getI(m.getColumnSize());
		Matrix c = new Matrix(m);
		for (int i = 0; i < power; i++) {
			c = matrixMultip(c, m);
		}
		return c;
	}

	/**
	 * Function makes a tropical sum with a Matrix.
	 * 
	 * @param first
	 *            a Matrix.
	 * @param scalar
	 *            a scalar.
	 * @return returns a Matrix after operation.
	 */
	public static Matrix matrixScalarMult(Matrix first, double scalar) {
		Matrix result = new Matrix(first.getRowSize(), first.getColumnSize());
		for (int i = 0; i < result.getRowSize(); i++) {
			for (int j = 0; j < result.getColumnSize(); j++) {
				result.setValueAt(first.getValueAt(i, j) + scalar, i, j);
			}
		}
		return result;
	}

	/**
	 * Function return an asterate A* of matrix A (matrix should be square) A* =
	 * I oplus A oplus .... oplus A^(n-1)
	 * 
	 * @param m
	 *            matrix A to be consider
	 * @return return an asterate A* of given matrix A
	 */
	public static Matrix getAsterate(Matrix m) {

		Matrix aster = new Matrix(m.getRowSize(), m.getColumnSize());
		Matrix I = getI(m.getRowSize());
		aster = matrixSum(I, m);
		Matrix powe = matrixMultip(m, m);
		for (int i = 0; i < aster.getColumnSize() - 1; i++) {
			aster = matrixSum(aster, powe);
			powe = matrixMultip(powe, m);
		}

		return aster;
	}

	/**
	 * Returns a multiplicative conjugate transponse of given matrix A. A' is a
	 * transponse such as (a' ij = - a ji)
	 * 
	 * @param thism
	 *            - given matrix (M x N).
	 * @return multiplicative conjugate transponse of given matrix.
	 */
	public static Matrix multipConjugateTranspose(Matrix thism) {
		Matrix z = new Matrix(thism.getColumnSize(), thism.getRowSize());
		for (int i = 0; i < thism.getRowSize(); i++) {
			for (int j = 0; j < thism.getColumnSize(); j++) {
				if (!Double.isInfinite(thism.getValueAt(i, j))) {
					z.setValueAt(-thism.getValueAt(i, j), j, i);
				}
			}
		}
		return z;
	}
}
