package Tropical.Mathematics;

import java.io.Serializable;
import java.util.Random;

/**
 * 
 * @author alex
 * @author gooogleshandi@gmail.com
 * 
 */

public class Matrix implements Cloneable, Serializable {
	/**
	 * Main class we working with. All values except scalars ar vectors or
	 * matrices. Vectors here represents as matrices with row count or column
	 * count equal to one.
	 * 
	 * @param rowSize
	 *            the row count of Matrix(vector).
	 * @param columnSize
	 *            the column count of Matrix(vector).
	 */
	private int rowSize, columnSize;
	private double[][] valueTable;

	/**
	 * Return a value in Matrix in particular cell
	 * 
	 * @param rowIndex
	 *            row index of cell
	 * @param columnIndex
	 *            column index of cell
	 * @return Matrix[rowIndex][columnIndex] value.
	 * @throws IndexOutOfBounds
	 *             exception.
	 */
	public double getValueAt(int rowIndex, int columnIndex) {
		return valueTable[rowIndex][columnIndex];
	}

	/**
	 * @return Returns the description of Matrix.
	 */
	@Override
	public String toString() {
		String S = String.format("Matrix is %d x %d format\r\n", rowSize,
				columnSize);
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				S += valueTable[i][j] + " ";
			}
			S += "\r\n";
		}
		return S;
	}

	/**
	 * A main constructor. returns the link to Matrix with rowSize count of rows
	 * and columnSize count of columns. All cells are by default filled with
	 * tropical zeros (equals to "negative infinity").
	 * 
	 * @param rowSize
	 *            count of rows in Matrix.
	 * @param columnSize
	 *            count of columns in Matrix.
	 */
	public Matrix(int rowSize, int columnSize) {
		valueTable = new double[rowSize][columnSize];
		this.rowSize = rowSize;
		this.columnSize = columnSize;
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				valueTable[i][j] = Double.NEGATIVE_INFINITY;
			}
		}
	}

	/**
	 * A main constructor. returns the link to Matrix with rowSize count of rows
	 * and columnSize count of columns. All cells are by default filled with
	 * with random integer positive values.
	 * 
	 * @param rowSize
	 *            count of rows in Matrix.
	 * @param columnSize
	 *            count of columns in Matrix.
	 * @param randomSize
	 *            a amplitude of set of positive values.
	 */
	public Matrix(int rowSize, int columnSize, int randomSize) {
		this(rowSize, columnSize);
		Random r = new Random();
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				valueTable[i][j] = r.nextInt(randomSize);
			}
		}
	}

	/**
	 * 
	 * A main constructor. returns the link to Matrix with rowSize count of rows
	 * and columnSize count of columns. All cells are by default filled with
	 * with random integer positive and negative values.
	 * 
	 * @param rowSize
	 *            count of rows in Matrix.
	 * @param columnSize
	 *            count of columns in Matrix.
	 * @param randomSize
	 *            a amplitude of set of positive values.
	 * @param hasNegative
	 *            if true Matrix has negative integer values. If false
	 *            constructor equals to @see
	 *            {@link Matrix#Matrix(int, int, int)} .
	 * 
	 */
	public Matrix(int rowSize, int columnSize, int randomSize,
			boolean hasNegative) {
		this(rowSize, columnSize);
		Random r = new Random();
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				if (r.nextInt(randomSize) % 2 == 0 || !hasNegative) {
					valueTable[i][j] = r.nextInt(randomSize);
				} else {
					valueTable[i][j] = -r.nextInt(randomSize);
				}
			}
		}
	}

	public Matrix(Matrix a) {
		valueTable = new double[a.getRowSize()][a.getColumnSize()];
		for (int i = 0; i < a.getRowSize(); i++) {
			for (int j = 0; j < a.getColumnSize(); j++) {
				valueTable[i][j] = a.getValueAt(i, j);
			}
		}
	}

	/**
	 * Sets a double value in cell defined by row and column index.
	 * 
	 * @param value
	 *            this value sets in defined cell. The "tropical zero" must be
	 *            defined as NEGATIVE_INFINITY
	 * @param rowIndex
	 *            this is row position of cell.
	 * @param columnIndex
	 *            this is column position of cell.
	 */

	public void setValueAt(double value, int rowIndex, int columnIndex) {
		valueTable[rowIndex][columnIndex] = value;
	}

	/**
	 * Sets a specific count of rows in Matrix. Usually decreasing. RowSize must
	 * be positive.
	 * 
	 * @param rowSize
	 *            the wanted count of rows in Matrix.
	 */
	public void setRowSize(int rowSize, Double fillvalue) {
		double[][] valueTable2 = new double[rowSize][this.columnSize];
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < this.columnSize; j++) {
				if (i < this.rowSize) {
					valueTable2[i][j] = valueTable[i][j];
				} else {
					valueTable2[i][j] = fillvalue;
				}
			}
		}
		this.rowSize = rowSize;
		valueTable = valueTable2;

	}

	/**
	 * Sets a specific count of rows in Matrix. Usually decreasing.
	 * 
	 * @param columnSize
	 *            the wanted count of columns in Matrix.
	 */
	public void setColumnSize(int columnSize, Double fill) {
		double[][] valueTable2 = new double[rowSize][columnSize];
		for (int i = 0; i < rowSize; i++) {
			for (int j = 0; j < columnSize; j++) {
				if (j < this.columnSize) {
					valueTable2[i][j] = valueTable[i][j];
				} else {
					valueTable2[i][j] = fill;
				}
			}
		}
		this.columnSize = columnSize;
		valueTable = valueTable2;

	}

	/**
	 * Function returns a number of rows in Matrix.
	 * 
	 * @return a count of row in Matrix.
	 */

	public int getRowSize() {
		return rowSize;
	}

	/**
	 * Function returns a number of columns in Matrix.
	 * 
	 * @return a count of column in Matrix.
	 */
	public int getColumnSize() {
		return columnSize;
	}

	

	/**
	 * Returns norm ||A|| of matrix A.
	 * 
	 * @return norm of given matrix.
	 */
	public double getNorm() {
		double maxvalue = Double.NEGATIVE_INFINITY;
		for (double[] arr : valueTable) {
			for (double z : arr) {
				if (maxvalue < z) {
					maxvalue = z;
				}
			}
		}
		return maxvalue;
	}

	/**
	 * 
	 * @return an invert matrix A- of A. where a-ij = -aji.
	 */
	public Matrix getInvert() {
		Matrix m = new Matrix(this.getColumnSize(), this.getRowSize());
		for (int i = 0; i < this.getRowSize(); i++) {
			for (int j = 0; j < this.getColumnSize(); j++) {
				if (Double.isInfinite(this.getValueAt(i, j))) {
					m.setValueAt(this.getValueAt(i, j), j, i);
				} else {
					m.setValueAt(-this.getValueAt(i, j), j, i);
				}
			}
		}
		return m;

	}

	/**
	 * 
	 * @return a trasponse of given matrix.
	 */
	public Matrix getTransponse() {
		Matrix m = new Matrix(this.getColumnSize(), this.getRowSize());
		for (int i = 0; i < this.getRowSize(); i++) {
			for (int j = 0; j < this.getColumnSize(); j++) {
				m.setValueAt(this.getValueAt(i, j), j, i);
			}
		}
		return m;
	}
}
