package Tropical.ProjectScheduling;

import java.util.*;

import Tropical.Mathematics.Evaluations;
import Tropical.Mathematics.Matrix;

/**
 * 
 * @author alex
 * @author gooogleshandi at gmail dot com
 */
public class Scheduling {
	/**
	 * Computes just-in-time criteria of given Project or Matrices. All jobs
	 * must be finished at one time. 1^{T} yy^{-}1 -> min C x = y D x <= x Δ =
	 * (C D*(1^{T} C D*)^{-})^{-}1 x = alpha D*(1 C D*)^{-} y = alpha C D*(1 C
	 * D*)^{-}
	 * 
	 * @param a
	 *            - matrix Aij - minimum lag time between initiation of j 1*
	 *            activity and end of i activity.
	 * @param c
	 *            - matrix Cij - minimum lag time between initiation of j
	 *            activity and initiation of i activity.
	 * @param x
	 *            - result vector of activity's start time.
	 * @param y
	 *            - result vector of activity's stop time.
	 */
	public static List<Matrix> justInTime(Matrix a, Matrix c, Matrix g, Matrix h) {
		if (!checkConstraints(a, c, g, h)) {
			System.err.print("Somethnig wrong, the delta is greater then 1.");
			return null;
		}
		// checking wether Tr(C)<1
		double traceC = Evaluations.getBigTrace(c);
		if (traceC > 0) {
			System.out.println("Trace of C matrix should be <= 0");
			return null;
		}
		Matrix asterC = Evaluations.getAsterate(c);
		Matrix aAsterC = Evaluations.matrixMultip(a, asterC);
		Matrix iT = Evaluations.getVectorI(c.getColumnSize());// .getTransponse();
		Matrix iTAAsterC_ = Evaluations.matrixMultip(iT, aAsterC).getInvert();
		Matrix CiTAAsterC_ = Evaluations.matrixMultip(asterC, iTAAsterC_);
		Matrix ACiTAAsterC_ = Evaluations.matrixMultip(a, CiTAAsterC_);
		Matrix delta = Evaluations.matrixMultip(ACiTAAsterC_.getInvert(),
				Evaluations.getVectorI(ACiTAAsterC_.getRowSize()).getInvert());
		// assuming alpha = 0
		Matrix x = CiTAAsterC_;
		Matrix y = ACiTAAsterC_;
		LinkedList<Matrix> result = new LinkedList<Matrix>();
		result.add(x);
		result.add(y);
		return result;
	}

	/**
	 * Minimizing maximum total time of project. max y_{i} - min x_{i} -> min
	 * 
	 * @param a
	 *            - matrix Aij - minimum lag time between initiation of j
	 *            activity and end of i activity.
	 * @param c
	 *            - matrix Cij - minimum lag time between initiation of j
	 *            activity and initiation of i activity.
	 * @param x
	 *            - result vector of activity's start time.
	 * @param y
	 *            - result vector of activity's stop time.
	 */
	public static List<Matrix> makeSpan(Matrix a, Matrix c, Matrix g, Matrix h) {
		if (!checkConstraints(a, c, g, h)) {
			System.err.print("Somethnig wrong, the delta is greater then 1.");
			return null;
		}
		double delta = a.getNorm();
		Matrix one = Evaluations.getVectorI(a.getRowSize());
		Matrix C1 = Evaluations.matrixScalarMult(one, -delta);
		Matrix _1TC = Evaluations.matrixMultip(a, one.getTransponse());
		Matrix C1_1TC = Evaluations.matrixMultip(C1, _1TC);
		Matrix x = Evaluations.matrixSum(Evaluations.getI(a.getColumnSize()),
				C1_1TC);
		x = Evaluations.matrixMultip(x, g);
		Matrix y = Evaluations.matrixMultip(a, x);
		LinkedList<Matrix> result = new LinkedList<Matrix>();
		result.add(x);
		result.add(y);
		return result;
	}

	/**
	 * Computes maximum deviaton of activities given Project or Matrices.
	 * 
	 * @param a
	 *            - matrix Aij - minimum lag time between initiation of j
	 *            activity and end of i activity.
	 * @param c
	 *            - matrix Cij - minimum lag time between initiation of j
	 *            activity and initiation of i activity.
	 * @param x
	 *            - result vector of activity's start time.
	 * @param y
	 *            - result vector of activity's stop time.
	 * @return - two matrix X and Y start and end matrix
	 * 
	 */
	public static List<Matrix> maximumDeviation(Matrix a, Matrix c, Matrix g,
			Matrix h) {
		if (!checkConstraints(a, c, g, h)) {
			System.err.print("Somethnig wrong, the delta is greater then 1.");
			return null;
		}
		LinkedList<Matrix> result = new LinkedList<Matrix>();
		Matrix Aij = a;
		Matrix Cij = c;
		Matrix asterC = Evaluations.getAsterate(Cij);
		Matrix D = Evaluations.matrixMultip(Aij, asterC);
		Matrix tranD = Evaluations.multipConjugateTranspose(D);
		Matrix DtranD = Evaluations.matrixMultip(D, tranD);
		double delta = DtranD.getNorm();
		double[] columnsMax = new double[DtranD.getColumnSize()];
		double[] columnsNorm = new double[DtranD.getColumnSize()];
		for (int i = 0; i < DtranD.getColumnSize(); i++) {
			double maxD = Double.NEGATIVE_INFINITY;
			double maxDt = Double.NEGATIVE_INFINITY;
			for (int j = 0; j < DtranD.getRowSize(); j++) {
				maxD = Math.max(maxD, D.getValueAt(j, i));
				maxDt = Math.max(maxDt, tranD.getValueAt(i, j));
			}
			columnsMax[i] = maxD + maxDt;
			columnsNorm[i] = maxDt;
		}
		int k = 0;
		double m = columnsMax[0];
		for (int i = 1; i < columnsMax.length; i++) {
			if (m < columnsMax[i]) {
				k = i;
				m = columnsMax[i];
			}
		}
		int s = 0;
		m = Double.NEGATIVE_INFINITY;
		for (int i = 0; i < D.getRowSize(); i++) {
			double value = -D.getValueAt(i, k);
			if (value > m) {
				m = value;
				s = i;
			}
		}
		// assume alpha = 0
		Matrix u = new Matrix(Cij.getRowSize(), 1);
		for (int j = 0; j < D.getRowSize(); j++) {
			if (j != k) {
				if (Double.isInfinite(D.getValueAt(s, j))) {
					u.setValueAt(D.getValueAt(s, j), j, 0);
				} else {
					u.setValueAt(-D.getValueAt(s, j), j, 0);
				}
			} else {
				u.setValueAt(columnsNorm[j], j, 0);
			}
		}
		Matrix x = Evaluations.matrixMultip(asterC, u);
		Matrix y = Evaluations.matrixMultip(D, u);
		result.add(x);
		result.add(y);
		return result;
	}

	/**
	 * Optimizing full time of project. max(-xi+yi) -> min
	 * 
	 * @param a
	 *            - start finish constraints as always
	 * @param c
	 *            - start start constraints as always
	 * @param g
	 *            - the most early time of operation starting
	 * @param h
	 *            - the most late time of operation ending
	 * @return - two matrix X and Y start and end matrix
	 */
	public static List<Matrix> fullTimeOptimization(Matrix a, Matrix c,
			Matrix g, Matrix h) {
		LinkedList<Matrix> result = new LinkedList<Matrix>();
		if (checkConstraints(a, c, g, h) == true) {
			Matrix d = Evaluations.matrixMultip(g, h.getInvert());
			d = Evaluations.matrixMultip(d, a);
			d = Evaluations.matrixSum(Evaluations.getI(d.getRowSize()), d);
			int n = a.getRowSize();
			LinkedList<Matrix> CD = new LinkedList<Matrix>();

			for (int k = 1; k <= n; k++) {
				Matrix C = new Matrix(n, n);
				LinkedList<int[]> indices = new LinkedList<int[]>();
				for (int z = 1; z <= n - k; z++) {
					indices.addAll(getAllComp(z, k + 1));
				}
				if (n == k) {
					indices.addAll(getAllComp(0, k + 1));
				}
				SortedSet<int[]> ss = new TreeSet<int[]>(new Comp());
				ss.addAll(indices);
				indices.clear();
				indices.addAll(ss);
				for (int[] ind : indices) {
					Matrix c_ = Evaluations.getI(c.getRowSize());
					for (int i = 0; i < ind.length; i++) {
						System.out.print(ind[i]);
						for (int count = 0; count < ind[i]; count++) {
							c_ = Evaluations.matrixMultip(c_, c);
						}
						if (i == ind.length - 1) {// b
						} else {
							c_ = Evaluations.matrixMultip(c_, a);// ba
						}
					}
					System.out.println();
					C = Evaluations.matrixSum(C, c_);
				}// for indices
				CD.add(Evaluations.matrixMultip(C, d));
			}// for k
			Double maxfi = Double.NEGATIVE_INFINITY;
			for (Matrix m : CD) {
				double thistrace = Math.pow(Evaluations.getTrace(m),
						1. / (CD.indexOf(m) + 1));
				if (maxfi < thistrace) {
					maxfi = thistrace;
				}
			}
			Matrix S = Evaluations.matrixScalarMult(a, -maxfi);
			S = Evaluations.matrixSum(S, c);
			Matrix S2 = Evaluations.matrixMultip(S, S);
			Matrix asterS = Evaluations.getAsterate(S);
			Matrix AasterS = Evaluations.matrixMultip(a, asterS);
			Matrix hAasterS = Evaluations.matrixMultip(h.getInvert(), AasterS);
			hAasterS.getInvert();
			Matrix x = Evaluations.matrixMultip(asterS, g);
			Matrix y = Evaluations.matrixMultip(a, x);
			result.add(x);
			result.add(y);
		} else {
			System.err.print("Somethnig wrong, the delta is greater then 1.");
			return null;
		}
		return result;
	}

	/**
	 * Optimazing full time of project
	 * 
	 * @param a
	 *            - start finish constraints as always
	 * @param c
	 *            - start start constraints as always
	 * @param matrix
	 * @return - two matrix X and Y start and end matrix
	 */
	public static List<Matrix> plannedTerms(Matrix a, Matrix c, Matrix g,
			Matrix b) {
		LinkedList<Matrix> result = new LinkedList<Matrix>();

		Matrix x = Evaluations.matrixMultip(b.getInvert(), a);
		x = x.getInvert();
		Matrix delta = Evaluations.matrixMultip(a, x);
		delta = delta.getInvert();
		delta = Evaluations.matrixMultip(delta, b);
		if (delta.getRowSize() == 1 && delta.getColumnSize() == 1) {
			if (delta.getValueAt(0, 0) == 0) {
				Matrix y = Evaluations.matrixMultip(a, x);
				result.add(x);
				result.add(y);
			}
			return result;
		} else {
			return null;
		}
	}

	public static SortedSet<String> inputString(String arg) {
		String exampleString = arg; // или String exampleString = arg;
		char[] z = exampleString.toCharArray();
		LinkedList<String> working = new LinkedList<String>();
		working.add(String.valueOf(z[0]));
		// пирамидкой добавляем новый литерал
		for (int i = 1; i < z.length; i++) {
			working = sort(working, String.valueOf(z[i]));
		}
		System.out.println(working.size() + " " + working);

		// для проверки нет повторений и просто чтобы отсортировать
		// (иногда бывают повторения когда есть повторяющиеся
		// литералы)
		SortedSet<String> ss = new TreeSet<String>(working);
		// System.out.println(ss.size() + " " + ss);
		return ss;
	}

	public static LinkedList<String> sort(LinkedList<String> s, String letter) {
		LinkedList<String> ll = new LinkedList<String>();
		for (String str : s) {
			char[] workarray = str.toCharArray();
			for (int k = 0; k < str.length() + 1; k++) {
				StringBuffer sb = new StringBuffer();
				for (int i = 0; i < str.length(); i++) {
					if (i == k) {
						sb.append(letter);
					}
					sb.append(workarray[i]);
					if (k == str.length() && i == str.length() - 1) {
						sb.append(letter);
					}
				}
				ll.add(sb.toString());
			}

		}
		return ll;
	}

	int min = 0, k = 0;

	public static LinkedList<int[]> makedevision(int n, int k) {
		int[] array = new int[k];
		LinkedList<int[]> all = new LinkedList<int[]>();
		all = devision(n, n, 0, 0, array, all);
		return all;
	}

	public static LinkedList<int[]> devision(int N, int n, int min, int i,
			int[] array, LinkedList<int[]> all) {
		if (n < 0 || min == N)
			return null;
		array[i] = min;
		if (min != 0) {
			devision(N, n - min, min, i + 1, array, all);
		}
		devision(N, n - 1, min + 1, i, array, all);
		if (n == 0) {
			all.add(Arrays.copyOfRange(array, 0, i + 1));
			for (int j = 0; j <= i; ++j) {
				System.out.print(array[j] + " ");
			}
			System.out.println(" ");
		}
		return all;
	}

	/**
	 * @param strings
	 */
	public static LinkedList<int[]> getAllComp(int number, int k) {

		// int[] array = makedevision(5, 40);
		LinkedList<int[]> need = new LinkedList<int[]>();
		need = makedevision(number, 100);
		if (need == null) {
			need = new LinkedList<int[]>();
		}
		for (int[] i : need) {
			for (int ii : i)
				System.out.print(ii);
			System.out.println();

		}
		// порядок <n
		LinkedList<int[]> result = new LinkedList<int[]>();
		Iterator<int[]> it = need.iterator();
		while (it.hasNext()) {
			int[] n = it.next();
			int sum = 0;
			n = Arrays.copyOf(n, k);
			for (int i = 0; i < k; i++) {
				sum += n[i];
			}
			if (sum <= number) {
				result.add(n);
			}
		}
		int[] defalt = { number };
		int[] defalt2 = { 0 };
		defalt = Arrays.copyOf(defalt, k);
		defalt2 = Arrays.copyOf(defalt2, k);
		result.add(defalt);
		result.add(defalt2);
		SortedSet<String> ss = new TreeSet<String>();
		for (int[] d : result) {
			String s = "";
			for (int i : d) {
				s += i;
			}
			ss.addAll(inputString(s));
		}
		need.clear();
		for (String s : ss) {
			char[] sch = s.toCharArray();
			int[] r = new int[sch.length];
			for (int i = 0; i < sch.length; i++) {
				r[i] = Integer.valueOf(sch[i]) - 48;
			}
			need.add(r);
		}
		need.toArray();
		return need;
	}

	static boolean checkConstraints(Matrix a, Matrix c, Matrix g, Matrix h) {
		LinkedList<Matrix> result = new LinkedList<Matrix>();
		double traceOfC = Evaluations.getBigTrace(c);
		Matrix r = Evaluations.matrixMultip(a, Evaluations.getAsterate(c));
		r = Evaluations.matrixMultip(h.getInvert(), r);
		r = Evaluations.matrixMultip(r, g);
		Matrix delta = Evaluations.matrixScalarSum(r, traceOfC);
		double trace = Double.NEGATIVE_INFINITY;
		if (delta.getValueAt(0, 0) <= 1 && delta.getRowSize() == 1
				&& delta.getColumnSize() == 1) {
			return true;
		}
		return false;
	}

}
